<?php

/**
 * Loads a folder from the database.
 *
 * @param $fid
 *   The folders identifier.
 * @param $refresh
 *   Allows refresh of the folders memory.
 * @return
 *   Returns false on failure, folder object on success.
 */
function media_folder_load($fid, $refresh = false) {
  static $folders;

  // clear the cache
  if ($refresh || !isset($folders)) {
    $folders = array();
  }

  // check if valid fid
  if (!is_numeric($fid)) {
    return false;
  }
  
  // check cache
  if (array_key_exists($fid, $folders)) {
    return $folders[$fid];
  }

  // load from database
  $f = db_fetch_object(db_query("SELECT * FROM {media_folders} WHERE fid=%d", $fid));
  
  // invalid folder
  if (!$f) {
    return $f;
  }
  
  // notify modules of the loading
  media_invoke_folder('load', $f);
  
  // save this as cached
  $folders[$fid] = $f;

  return $folders[$fid];
}

/**
 * Checks for existance of a folder by the same as another in a folder.
 *
 * @param $name
 *   Name for folder.
 * @param $fid
 *   The parent folder identifier.
 * @param $ignore
 *   Gives us the ability to ignore an edited item
 * @return
 *   Returns true on success, false on failure.
 */
function media_folder_exists($name, $fid, $ignore) {
  return (db_result(db_query("SELECT fid FROM {media_folders} WHERE name='%s' AND parent=%d AND fid!=%d", $name, $fid, $ignore)) !== false);
}

/**
 * Moves the folders to the correct location.
 *
 * @param $src_id
 *   The source folder identifier.
 * @param $dest_id
 *   The destination folder identifier.
 * @return
 *   Return false on failure, true on success.
 */
function media_folder_move($src_id, $dest_id, $workspace = 'media') {
  // load folders
  if (!($source = media_folder_load($src_id)) || !($dest = media_folder_load($dest_id))) {
    return false;
  }

  // no need to move it, if it is the same directory, or creates circular connection
  if (($source->fid && $dest->lineage && strpos('/'. $source->fid .'/', $dest->lineage) !== false)) {
    return false;
  }

  // check for existance of folder already in that folder with same name
  if (db_result(db_query("SELECT fid FROM {media_folders} WHERE parent=%d AND name='%s' AND fid != %d", $dest->fid, $source->name, $src_id))) {
    mapi_error(t('Directory of that name already exists.'), 'media');
    return false;
  }

  // update the actual folder
  $lineage = $dest->lineage . ($dest->lineage ? '' : '/') ."{$dest->fid}/";
  $depth = $dest->depth + 1;
  $path = $dest->path . $source->name .'/';
  db_query("UPDATE {media_folders} SET parent=%d, lineage='%s', depth=%d, path='%s' WHERE fid=%d", $dest->fid, $lineage, $depth, $path, $source->fid);

/*

  // otherwise, cycle through all child folders
  $results = db_query("SELECT * FROM {media_folders} WHERE lineage LIKE '%s/%%'", $source->lineage);
  while ($obj = db_fetch_object($results)) {
    $lineage = str_replace("/{$source->fid}/", "/{$dest->fid}/", $obj->lineage);
    $depth = count(explode('/', trim($lineage, '/')));
    db_query("UPDATE {media_folders} SET lineage='%s', depth=%d WHERE fid=%d", $lineage, $depth, $obj->fid);
  }
*/
  $path = media_directory_public() . $path;
  
  // make sure the path exists
  if (!mapi_directory_check($path, true, false, $workspace)) {
    return FALSE;
  }

  // clear the caches
  media_folder_load(null, true);
  media_item_folder(null, $workspace, true);

  // move all the media paths to reflect the shifts.
  $results = db_query("SELECT path, name, ext FROM {media} WHERE fid = %d", $source->fid);
  while ($obj = db_fetch_object($results)) {
    $filename = media_item_filename($obj);
    if (!mapi_file_move($filename, $path . $obj->name .'.'. $obj->ext)) {
      mapi_error(t('Unable to move file (%source) to (%dest)', array('%source' => $filename, '%dest' => $path . $obj->name .'.'. $obj->ext)));
    }
  }
  db_query("UPDATE {media} SET path='%s' WHERE fid=%d", $path, $source->fid);

  // move subfolders
  $results = db_query("SELECT fid FROM {media_folders} WHERE parent = %d", $source->fid);
  while ($obj = db_fetch_object($results)) {
    media_folder_move($obj->fid, $source->fid, $workspace);
  }
  
  // clear other caches
  cache_clear_all('*', 'cache_menu', true);
  _media_folder_cache(true);
  
  // unlink the directory, there shouldn't be anything left of it.
  mapi_directory_delete(media_directory_public() . $source->path, false, $workspace);
  
  // notify modules of change to folder
  $source->old_parent  = $source->parent;
  $source->old_lineage = $source->lineage;
  $source->old_depth   = $source->depth;
  $source->old_path    = $source->path;
  $source->parent      = $dest->fid;
  $source->lineage     = $lineage;
  $source->depth       = $depth;
  $source->path        = $dest->path . $source->name .'/';
  media_invoke_folder('move', $source);

  return true;
}

/**
 * Renames the folder.
 *
 * @param $src_id
 *   The folder identifier.
 * @param $name
 *   The new name to rename it to.
 * @return
 *   Return false on failure, true on success.
 */
function media_folder_rename($src_id, $name, $workspace = 'media') {
  // load the folder
  if (!($source = media_folder_load($src_id))) {
    return false;
  }
  
  // don't need to change it, if the name is the same
  if ($source->name == $name) {
    return true;
  }

  // There a folder of the same name in the same directory
  if (db_result(db_query("SELECT fid FROM {media_folders} WHERE parent=%d AND name='%s' AND fid != %d", $source->parent, $name, $source->fid))) {
    mapi_error(t('Directory of that name already exists'), 'media');
    return false;
  }

  // get the new path name
	$base = media_folder_load($source->parent);
  $path = $base->path . $name .'/';
	
  // physically rename it
  if (!mapi_directory_move(media_directory_public() . $source->path, media_directory_public() . $path, $workspace)) {
    mapi_error(t('Unable to rename directory (%source) to (%dest)', array('%source' => media_directory_public() . $source->path, '%dest' => media_directory_public() . $path)), 'media');
    return false;
  }

  // now change all the path names for the child folders
  $results = db_query("SELECT fid, path FROM {media_folders} WHERE path LIKE '%s%%'", $source->path);
  while ($obj = db_fetch_object($results)) {
    // change the name, using preg_replace to guarantee the change from the beginning only
    $new_path = preg_replace('|^'. preg_quote($source->path) .'|', $path, $obj->path);

    // check that it's not the same directory
    if ($new_path != $obj->path) {
      db_query("UPDATE {media_folders} SET path='%s' WHERE fid=%d", $new_path, $obj->fid);
    }
  }

  // change the name of the folder, and the path it represents
  db_query("UPDATE {media_folders} SET name='%s', path='%s' WHERE fid=%d", $name, $path, $source->fid);
  
  // clear the media_item_folder cache
  media_item_folder(null, $workspace, true);
  cache_clear_all('*', 'cache_menu', true);
  _media_folder_cache(true);
  
  $source->old_name = $source->name;
  $source->name = $name;
  $source->path = $path;
  media_invoke_folder('rename', $source);

  return true;
}

/**
 * Creates a folder within the parent directory.
 *
 * @param $parent
 *   The folder identifier for the parent of the to be created directory.
 * @param $name
 *   The name of the folder.
 * @return
 *   Return false on failure, a folder object on success.
 */
function media_folder_create($parent, $name, $workspace = 'media') {
  global $user;

  // load parent, if exists
  if (!($source = media_folder_load($parent))) {
    return false;
  }

  // check if directory does not already exist
  if (db_result(db_query("SELECT fid FROM {media_folders} WHERE parent=%d AND name='%s'", $source->fid, $name))) {
    mapi_error(t('Folder of that name already exists.'), 'media');
    return false;
  }

  // We need to make sure that the paths are always relative paths, i.e. no / in front.
  // create the tree system
  $path = $source->path . $name .'/';

  // create the hierarchical path
  if (!mapi_directory_create(media_directory_public() . $path, false, $workspace)) {
    mapi_error(t('Unable to create directory (%dir)', array('%dir' => media_directory_public() . $path)), 'media');
    return false;
  }

  // create the correct lineage
  $lineage = $source->lineage . ($source->lineage ? '' : '/') ."{$source->fid}/";
  
  // set folder data and
  $fid = db_next_id("{media_folders}_fid");
  $folder = array('fid' => $fid, 'uid' => $user->uid, 'parent' => $source->fid,
    'name' => $name, 'depth' => $source->depth + 1, 'lineage' => $lineage, 
    'path' => $path
  );
  $folder_values = array('fid' => '%d', 'uid' => '%d', 'parent' => '%d',
    'name' => "'%s'", 'depth' => '%d', 'lineage' => "'%s'", 'path' => "'%s'"
  );

  // add this folder to the database
  db_query('INSERT INTO {media_folders} ('. implode(', ', array_keys($folder)) .') VALUES ('. implode(', ', $folder_values) .')', $folder);

  cache_clear_all('*', 'cache_menu', true);
  _media_folder_cache(true);

  // notify the modules of the creation of the folder
  $folder = (object)$folder;
  media_invoke_folder('create', $folder);
  
  // this notifies on load
  $folder = media_folder_load($fid);

  return $folder;
}

/**
 * Creates a folder within the parent directory.
 *
 * @param $src_id
 *   The folder identifier.
 * @return
 *   Return false on failure, true on success.
 */
function media_folder_delete($fid, $delete = false, $workspace = 'media') {
  // load the folder to delete
  if (!($source = media_folder_load($fid))) {
    return false;
  }

  // go through all children folders
  $results = db_query("SELECT fid FROM {media_folders} WHERE lineage LIKE '%%/%d/%%' ORDER BY depth DESC", $source->fid);
  while ($obj = db_fetch_object($results)) {
    media_folder_delete($obj->fid, $delete, $workspace);
  }

  // only move the media if there is something to move
  $test = db_result(db_query("SELECT COUNT(*) FROM {media} WHERE fid=%d", $source->fid));
  if ($test) {
    // got through all media located in folder
    $results = db_query("SELECT mid FROM {media} WHERE fid=%d", $source->fid);
    while ($obj = db_fetch_object($results)) {
      // delete if that's the policy
      if ($delete) {
        media_item_delete($obj->mid, $workspace);
      }
      // otherwise move to parent folder
      else {
        media_item_move($obj->mid, $source->parent, $workspace);
      }
    }
  }

  // update database
  db_query("DELETE FROM {media_folders} WHERE fid=%d", $source->fid);

  // clear the media_item_folder cache
  media_item_folder(null, $workspace, true);
  cache_clear_all('*', 'cache_menu', true);
  _media_folder_cache(true);
  
  // notify the modules of folder change
  media_invoke_folder('delete', $source);

  // remove the physical folder 
  if (!mapi_directory_delete(media_directory_public() . $source->path, false, $workspace)) {
    mapi_error(t('Unable to delete directory (%directory).', array('%directory' => media_directory_public() . $source->path)), 'media');
  }
  
  return true;
}

/**
 * Creates the list of subfolders for a folder
 *
 * @param $fid
 *   The folder identifier.
 * @param $uid
 *   Specifies the user id which has access to these subfolders. 0 means all accessible.
 * @return
 *   Returns an associative array, using the sub-folder identifier to reference the name.
 */
function media_folder_subfolders($fid) {
  $children = array();

  $folders = _media_folder_cache();
  if (array_key_exists($fid, $folders)) {
    $results = db_query("SELECT fid FROM {media_folders} WHERE fid != parent AND parent=%d", $fid);
    while ($obj = db_fetch_object($results)) {
      if (array_key_exists($obj->fid, $folders)) {
        $children[$obj->fid] = $folders[$obj->fid]['name'];
      }
    }
  }
  return $children;
}

/**
 * Returns the items contained within the folder, based on permissions for user.
 *
 * @param $fid
 *   The folder identifier.
 * @param $uid
 *   Specifies the user identifer for the 
 * @return
 *   Returns an associative array, using the sub-folder identifier to reference the name.
 */
function media_folder_items($fid, $uid = 0) {
  $media = array();
  $results = db_query("SELECT mid, name FROM {media} WHERE fid=%d" . ($uid ? ' AND uid=%d' : ''), $fid, $uid);
  while ($obj = db_fetch_object($results)) {
    $media[$obj->mid] = $obj->name;
  }
  return $media;
}

/**
 * Finds the FID of a folder with specific path.
 *
 * @param $path
 *   The folder path.
 * @return
 *   Returns folder object or false.
 */
function media_folder_path($path) {
  $path = str_replace('\\', '/', $path);
  if (!($fid = db_result(db_query("SELECT fid FROM {media_folders} WHERE path='%s/'", rtrim($path, '/'))))) {
    return false;
  }
    
  return media_folder_load($fid);
}

/**
 * Returns a list of the children within a folder.
 *
 * @param $fid
 *   The folder identifier.
 * @param $paging
 *   Provides both for paging and what element is currently in use.
 * @return
 *   Returns array of media identifier associated names of the entire tree 
 */
function media_folder_children($fid, $paging = false) {
  $children = array();
  $sql = "SELECT m.mid, m.title FROM {media} m WHERE fid=%d ORDER BY m.title ASC";
  $results = ($paging ? pager_query($sql, variable_get('media_folder_page', 10), $paging, null, array($fid)) : db_query($sql, $fid));
  while ($obj = db_fetch_object($results)) {
    $children[$obj->mid] = $obj->title;
  }
  return $children;
}

/**
 * Returns the entire subtree as a flat array of folder identifiers and names.
 *
 * @param $fid
 *   The folder identifier.
 * @return
 *   Returns array of folder identifier associated names of the entire tree 
 *   including the folder itself.
 */
function media_folder_branches($fid) {
  $branches = array();
  $folders = _media_folders_cache();
  if (array_key_exists($fid, $folders)) {
    $results = db_query("SELECT fid FROM {media_folders} WHERE lineage LIKE '%%/%d/%%' OR fid=%d ORDER BY depth, name ASC", $fid, $fid);
    while ($obj = db_fetch_object($results)) {
      if (array_key_exists($obj->fid, $folders)) {
        $branches[$obj->fid] = $folders[$obj->fid]['name'];
      }
    }
  }
  
  return $branches;
}

/**
 *  Returns the fid of the parent folder.
 *  
 *  @param $fid
 *    Folder ID.
 *  @return
 *    Returns either the parent folder ID, or FALSE if none exists.
 */
function media_folder_parent($fid) {
  return db_result(db_query("SELECT parent FROM {media_folders} WHERE fid=%d", $fid));
}

/**
 *  Provide a list of the parents of the given FID
 *
 *  @param $fid
 *    Folder ID.
 *  @return
 *    Returns an array with a list of the parent FIDs.
 */
function media_folder_parents($fid) {
  $lineage = db_result(db_query("SELECT lineage FROM {media_folders} WHERE fid=%d", $fid));
  if (!$lineage) {
    return array();
  }
  // remove all the extras after exploding into an array
  return array_filter(explode('/', $lineage));
}

/**
 *  Determines whether folder is a direct child of a given parent.
 *
 *  @param $fid
 *    Folder ID.
 *  @return
 *    Returns TRUE or FALSE.
 */
function media_folder_ischild($fid, $pid) {
  return ($pid === db_result(db_query("SELECT parent FROM {media_folders} WHERE fid=%d", $fid)));
}

/**
 *  Determines whether folder is a branch of a given parent.
 *
 *  @param $fid
 *    Folder ID.
 *  @return
 *    Returns TRUE or FALSE.
 */
function media_folder_isbranch($fid, $pid) {
  $parents = media_folder_parents($fid);
  return in_array($pid, $parents);
}

/**
 *  Generates a cached list of folders for the menu system, for the current
 *  user.
 */
function media_folder_menu_tree() {
  $folders = _media_folder_cache();
  $details = array();
  foreach ($folders['line'] as $line => $fid) {
    $details[$fid] = array('line' => $line, 'name' => $folders[$fid]['name']);
  }
  return $details;
}

function media_folder_menu_id($lineage) {
  $folders = _media_folder_cache();
  
  $lineage = array_filter(explode('/', $lineage));
  if (isset($folders['line'])) {
    // attempt to find the base within the lineage pathways
    while (count($lineage) > 0) {
      if (array_key_exists(implode('/', $lineage) .'/', $folders['line'])) {
        return $folders['line'][implode('/', $lineage) .'/'];
      }
      array_pop($lineage);
    }
  }
  return false;  
}

/**
 *   Loads or generates the cached version of the media folders.
 *   The depth calculated
 */

function _media_folder_cache($refresh = false) {
  static $folders;
  global $user;

  if (!isset($folders)) {
    $cache = cache_get('media_folders:' . $user->uid, 'cache_media');
    if ($cache->data) {
      $folders = unserialize($cache->data);
    }
  }
  
  if ($refresh || !isset($folders)) {
    // create basic folders with defaults of hardcoded folder 0.
    $folders = array(
      0 => array('name' => t('none'), 'line' => ''),
      'line' => array(FOLDER_PATH .'/'  => 0)
    );

    // what depth to load up to
    $sql = "SELECT fid, name, path FROM {media_folders} WHERE fid != 0";
    $order = ' ORDER BY depth, name ASC';

    // allows check by folder
    if (user_access('view media folder') || $user->uid == 1) {
      $results = db_query($sql);
    }
    // allows check by own folder
    elseif (user_access('view own media folder')) {
      $results = db_query($sql .' AND uid=%d', $uid);
    }
    
    // load all visible objects
    while ($object = db_fetch_object($results)) {
      $folders[$object->fid] = array('name' => $object->name, 'line' => $object->path);
      $folders['line'][FOLDER_PATH .'/'. $object->path] = $object->fid;
    }
    // make sure they are in the correct ordering
    ksort($folders['line']);
    
    // set the cache for the user
    cache_set('media_folders:' . $user->uid, 'cache_media', serialize($folders), CACHE_TEMPORARY);
  }
  return $folders;  
}
