<?php

/**
 * Loads a media item.
 *
 * @param $mid
 *   The media identifier from the database.
 * @param $refresh
 *   Refreshes the memory list.
 * @return
 *   Returns false on failure, media object on success.
 */
function media_item_load($mid, $refresh = false) {
  static $media;

  // refresh the cache
  if (!isset($media) || (empty($mid) && $refresh)) {
    $media = array();
  }

  if (!is_numeric($mid)) {
    return false;
  }

  // if media cached
  if (array_key_exists($mid, $media) && !$refresh) {
    return $media[$mid];
  }

  // load from database
  $m = db_fetch_object(db_query("SELECT * FROM {media} WHERE mid=%d", $mid));

  // not a valid item
  if (!$m) {
    return $m;
  }

  // unserialize stored data
  $m->dimensions = unserialize($m->dimensions);

  // notify others that the media is deleted
  media_invoke_item('load', $m);

  // save into cache
  $media[$mid] = $m;

  // return object
  return $media[$mid];
}

/**
 * Inserts a media item.
 *
 * @param $media
 *   The media upload object
 * @param $fid
 *   The folder id for the media object.
 * @return
 *   Returns false on failure, media object on success.
 */
function media_item_insert($media, $fid = 0, $workspace = 'media') {
  // creates a media item from filename
  global $user;

  // check that some basics are actually set
  if (!is_object($media) || !($media->title) || (!is_file($media->path) && !is_file($media->path . $media->name .'.'. $media->ext))) {
    return false;
  }

  // load parent to check
  if (!($parent = media_folder_load($fid))) {
    return false;
  }

  // find a unique hash for the media.
  do {
    $hash = md5($media->path . chr(rand()));
    $result = db_query("SELECT hash FROM {media} WHERE hash='%s'", $hash);
    $obj = db_fetch_object($result);
  } while ($obj);

  // determine path
  $path = media_invoke_item('path', $media);
  // default to public directory with path
  if (empty($path)) {
    $path = media_directory_public() . $parent->path;
  }
  // otherwise take the first path off the array
  else {
    $path = array_shift($path);
  }

  // make sure it exists, within workspace
  if (!mapi_directory_check($path, true, false, $workspace)) {
    mapi_error(t('Unable to create directory !path', array('!path' => $path)), 'media');
    return false;
  }

  // get the file details
  $name = (!empty($media->name) ? $media->name : mapi_file_name($media->path));
  $ext  = (!empty($media->ext)  ? $media->ext : mapi_file_ext($media->path));
  $fname = $name .'.'. $ext;

  // if path is the only thing set, then change $media->path accordingly
  if (empty($media->name)) {
    $media->path = dirname($media->path);
  }

  // copy the file
  if (!($file = mapi_file_copy($media->path . $fname, $path . $fname, FILE_EXISTS_RENAME, $workspace))) {
    mapi_error(t('Unable to copy !filename to !path', array('!filename' => $media->path . $fname, '!path' => $path . $fname)));
    return false;
  }

  // check for properly filled out values
  $media->mid         = db_next_id("{media}_mid");
  $media->title       = ($media->title ? $media->title : t('Untitled'));
  $media->name        = mapi_file_name($file);
  $media->ext         = mapi_file_ext($file);
  $media->mime        = ($media->mime ? $media->mime : mapi_extension_mime($media->ext));
  $media->dimensions  = ($media->dimensions ? $media->dimensions : mapi_file_dimensions($file));
  $media->type        = ($media->type ? $media->type : mapi_extension_type($media->ext));
  $media->path        = $path;
  $media->created     = ($media->created ? $media->created : time());
  $media->changed     = time();
  $media->expire     = ($media->expire ? $media->expire : 0);

  // define the media and it's values to insert into the database
  $mid = $media->mid;
  $media_objects = array('mid' => $mid, 'fid' => $parent->fid, 'uid' => $user->uid,
    'title' => $media->title, 'name' => $media->name, 'mime' => $media->mime, 'hash' => $hash,
    'dimensions' => serialize($media->dimensions), 'type' => $media->type, 'ext' => $media->ext,
    'created' => $media->created, 'changed' => $media->changed, 'path' => $media->path,
    'summary' => $media->summary, 'source' => $media->source, 'reference' => $media->reference,
    'expire' => $media->expire,
  );
  $media_values = array('mid' => '%d', 'fid' => '%d', 'uid' => '%d',
    'title' => "'%s'", 'name' => "'%s'", 'mime' => "'%s'", 'hash' => "'%s'",
    'dimensions' => "'%s'", 'type' => "'%s'", 'ext' => "'%s'",
    'created' => '%d', 'changed' => '%d', 'path' => "'%s'",
    'summary' => "'%s'", 'source' => "'%s'", 'reference' => "'%s'",
    'expire' => '%d',
  );

  // add this folder to the database
  if (db_query('INSERT INTO {media} (' . implode(', ', array_keys($media_objects)) . ') VALUES (' . implode(', ', $media_values) . ')', $media_objects)) {
    watchdog('media', t('Created media') .':('. $mid .') ' . $media->path . $media->name .'.'. $media->ext);
  }
  // otherwise return false with error
  else {
    mapi_error(t('Unable to insert media into database') .':('. $mid .') ' . $media->path);
    return false;
  }

  // notify others that the media as been created
  media_invoke_item('create', $media);

  // reload the item, thereby giving the correct details from the database
  $media = media_item_load($mid);

  // return the newly created media
  return $media;
}

/**
 * Creates a media item from a filename.
 *
 * @param $filename
 *   The filename to copy.
 * @param $fid
 *   The folder identifier for the media item.
 * @return
 *   Returns false on failure, media object on success.
 */
function media_item_create($filename, $title, $parent = 0, $workspace = 'media') {
  // must have title and a filename that exists
  if (!$title || !is_file($filename)) {
    return false;
  }

  // create an object which can be processed correctly
  $media = new stdClass;
  $media->title = $title;
  $media->name = mapi_file_name($filename);
  $media->ext = mapi_file_ext($filename);
  $media->path = dirname($filename) .'/';

  return media_item_insert($media, $parent, $workspace);
}

/**
 * Deletes an item from the database.
 *
 * @param $mid
 *   The identifier for the media item.
 */
function media_item_delete($mid, $workspace = null) {
  // load item from database
  if (!($media = media_item_load($mid))) {
    return false;
  }

  $path = $media->path . $media->name .'.'. $media->ext;
  // delete the original
  if (!mapi_file_delete($path, $workspace)) {
    mapi_error(t('Failed to delete media original (%file).', array('%file' => $media->path . $media->name . '.'. $media->ext)));
  }

  // makes sure the generated derivatives for this media are removed
  if (module_exists('mapi_derivatives')) {
    mapi_derivative_flush_path($path);
  }

  // remove the media
  db_query("DELETE FROM {media} WHERE mid=%d", $media->mid);

  // remove common metadata
  db_query("DELETE FROM {media_extras} WHERE mid=%d", $media->mia);

  // remove the extra details that are associated with the media
  db_query("DELETE FROM {media_metadata} WHERE mid=%d", $media->mid);

  // notify others that the media is deleted
  media_invoke_item('delete', $media);
}

/**
 * Renames an item, including original and derivatives.
 *
 * @param $mid
 *   The identifier for the media item.
 * @param $name
 *   The new name, invalid characters filtered.
 * @return
 *   Return false on failure, true on success
 */
function media_item_rename($mid, $name, $workspace = 'media') {
  // load derivative
  if (!($media = media_item_load($mid))) {
    return false;
  }

  // replace potential invalid characters with '-'
  $name = preg_replace('/[^\w\d \-_\(\)\[\]\"\']+/i', '-', $name);

  // update the media settings
  $old_media = drupal_clone($media);
  $media->name = $name;

  $source = media_item_filename($old_media);
  $dest = media_item_filename($media);

  // rename the file on disk
  if ($file = mapi_file_move($source, $dest, FILE_EXISTS_RENAME, $workspace)) {
    $media->name = mapi_file_basename($file);
  }
  else {
    mapi_error(t('Unable to rename file (%source) to (%dest)', array('%source' => $source, '%dest' =>$dest)));
    return FALSE;
  }

  db_query("UPDATE {media} SET name='%s' WHERE mid=%d", $media->name, $media->mid);

  media_item_load($media->mid, true);

  // notify others that the media has been renamed
  $media->old_name = $old_media->name;
  media_invoke_item('rename', $media);

  return true;
}

/**
 * Moves an item from one folder to another. This moves the derivatives if the media system is
 * set to a hierarchical structure.
 *
 * @param $mid
 *   The identifier for the media item.
 * @param $fid
 *   The new folder to move media item to.
 * @return
 *   Return false on failure, true on success
 */
function media_item_move($mid, $fid, $workspace = 'media') {
  // load the media and folder
  if (!($media = media_item_load($mid)) || !($folder = media_folder_load($fid))) {
    return false;
  }

  // no real need to shift it
  if ($media->fid == $folder->fid) {
    return true;
  }

  // update the media settings
  $old_media = drupal_clone($media);
  $media->fid = $folder->fid;

  // rename the media itself
  $source = $media->path . $media->name .'.'. $media->ext;

  // change the folder path
  $media->path = media_item_folder($media, $workspace, true);
  $dest = media_item_filename($media);

  if ($file = mapi_file_move($source, $dest, FILE_EXISTS_RENAME, $workspace)) {
    watchdog('media', t('Moved file from (%source) to (%dest)', array('%source' => media_directory_public() . $source, '%dest' => media_directory_public() . $dest)));
    db_query("UPDATE {media} SET name='%s', path='%s' WHERE mid=%d", mapi_file_name($file), $media->path, $media->mid);
  }
  else {
    mapi_error(t('Unable to rename file (%source) to (%dest)', array('%source' => $source, '%dest' =>$dest)));
    return FALSE;
  }

  // change the media folder
  db_query("UPDATE {media} SET fid=%d, path='%s' WHERE mid=%d", $fid, $media->path, $media->mid);

  media_item_load($media->mid, true);

  $media->old_ext = $old_media->ext;
  $media->old_name = $old_media->name;
  $media->old_path = $old_media->path;
  $media->old_filename = media_item_filename($old_media);
  $media->old_fid = $old_media->fid;

  // notify others that the media has been moved
  media_invoke_item('move', $media);

  return true;
}

/**
 * Loads the metadata for the item.
 *
 * @param $mid
 *   The media identifier.
 * @return
 *   Returns an associative key array for the item.
 */
function media_metadata_load($mid, $refresh = false) {
  static $media;

  $metadata = array();
  if (!isset($media) || (empty($mid) && $refresh)) {
    $media = array();
  }
  if (array_key_exists($mid, $media) && !$refresh) {
    return $media[$mid];
  }

  // load inbuilt metadata
  $inbuilt = array('title', 'summary', 'source', 'reference', 'created', 'expire');
  $m = media_item_load($mid);
  foreach ($inbuilt as $key) {
    $metadata[$key] = $m->$key;
  }

  // load uncommonly used metadata
  $results = db_query("SELECT * FROM {media_metadata} WHERE mid=%d", $mid);
  while ($obj = db_fetch_object($results)) {
    $value = unserialize($obj->value);
    $metadata[$obj->type] = ($value !== false ? $value : $obj->value);
  }

  // load commonly used metadata
  $extras = db_fetch_array(db_query("SELECT * FROM {media_extras} WHERE mid=%d", $mid));
  if (count($extras)) {
    $extras[$mid]['info'] = unserialize($extras[$mid]['info']);
    $metadata = array_merge($metadata, $extras);
  }

  // allow modules to modify their metadata
  $metadata['mid'] = $mid;
  media_invoke_metadata('load', $metadata);
  unset($metadata['mid']);

  $media[$mid] = $metadata;

  return $metadata;
}

/**
 * Saves the metadata for the item.
 *
 * @param $mid
 *   The media identifier.
 * @param $metadata
 *   The metadata information as an array
 */
function media_metadata_save($mid, $metadata) {
  if (is_array($metadata)) {
    // allow modules to modify their metadata, before saving.
    $metadata['mid'] = $mid;
    media_invoke_metadata('save', $metadata);
    unset($metadata['mid']);

    // the inbuilt metadata
    $inbuilt = array(
      'title' => $metadata['title'],
      'summary' => $metadata['summary'],
      'source' => $metadata['source'],
      'reference' => $metadata['reference'],
      'created' => $metadata['created'],
      'expire' => $metadata['expire']
    );

    // generate the correct query for saving the data, while removing from metadata
    $sql = array();
    foreach ($inbuilt as $key => $value) {
      $sql[] = $key ."= '%s'";
      unset($metadata[$key]);
    }

    // run update query
    $inbuilt[] = $mid;
    db_query("UPDATE {media} SET ". implode(', ', $sql) ." WHERE mid=%d", $inbuilt);

    // serialize the standard metadata
    $standard = array(
      'mid' => $mid,
      'licence' => (isset($metadata['licence']) ? $metadata['licence'] : null),
      'description' => (isset($metadata['description']) ? $metadata['description'] : null),
      'bitrate' => (isset($metadata['bitrate']) ? $metadata['bitrate'] : null),
      'quality' => (isset($metadata['quality']) ? $metadata['quality'] : null),
      'length' => $metadata['length'],
      'info' => serialize($metadata['info']),
    );

    // just in case something already exists
    db_query("DELETE FROM {media_extras} WHERE mid=%d", $mid);
    db_query("INSERT INTO {media_extras} (". implode(', ', array_keys($standard)) .") VALUES(%d,". implode(', ', array_pad(array(), count($standard)-1, "'%s'")) .")", $standard);

    // remove all the extra metadata which is not common.
    foreach (array_keys($standard) as $key) {
      unset($metadata[$key]);
    }

    // wipe unnecessary metadata
    $metadata = array_filter($metadata);

    $sql = array();
    $values = array();
    foreach ($metadata as $key => $value) {
      if (!is_numeric($value) && !is_string($value)) {
        $value = serialize($value);
      }
      $values[] = $mid;
      $values[] = $key;
      $values[] = $value;
      $sql[] = "(%d, '%s', '%s')";
    }

    // remove old ones
    db_query("DELETE FROM {media_metadata} WHERE mid=%d", $mid);

    // add new ones
    if (count($sql)) {
      db_query("INSERT INTO {media_metadata} (mid, type, value) VALUES ". implode(', ', $sql), $values);
    }

    media_metadata_load($mid, true);
  }
}

/**
 * Gets the list of metadata keys used within the database
 * This does not include the inbuilt metadata within the media table.
 *
 * @return
 *   Returns array of keys used for metadata within the database
 */
function media_metadata_used($refresh = false) {
  static $list = array();

  if (!count($list) || $refresh) {
    // standard metadata from the extras table
    $list = array('title', 'summary', 'source', 'reference', 'created', 'expire', 'licence', 'description', 'bitrate', 'quality', 'length', 'info');

    // get the generated detail information
    $results = db_query("SELECT DISTINCT(type) FROM {media_metadata} ORDER BY type ASC");
    while ($obj = db_fetch_object($results)) {
      $list[] = $obj->type;
    }
  }

  return $list;
}

/**
 * Checks for existance of a media file by the same as another in a folder.
 *
 * @param $name
 *   Name for media file.
 * @param $fid
 *   The folder identifier.
 * @param $ignore
 *   Gives us the ability to ignore an edited item
 * @return
 *   Returns true on success, false on failure.
 */
function media_item_exists($name, $fid, $ignore = false) {
  return (db_result(db_query("SELECT mid FROM {media} WHERE name='%s' AND fid=%d AND mid!=%d", $name, $fid, $ignore)) !== false);
}

/**
 * Returns the public filename for the item
 *
 * @param $media
 *   The media object.
 * @return
 *   Returns generated filename.
 */
function media_item_filename($media, $workspace = 'media') {
  if (!mapi_directory_check_location($media->path, $workspace)) {
    return 'media/'. $media->mid .'/transfer';
  }

  // returns nothing if media is not an object, and everything is empty.
  if (!is_object($media) || (!$media->path && !$media->name && !$media->ext)) {
    return '';
  }

  return $media->path . $media->name .'.'. $media->ext;
}

/**
 * Provides the folder path for the media
 *
 * @param $media
 *   A media object
 * @param $refresh
 *   Forces refresh of directory buffering.
 */
function media_item_folder($media, $workspace = 'media', $refresh = false) {
  static $paths;

  // free memory if necessary
  if ($refresh || !isset($paths)) {
    $paths = array();
  }

  // only works if the media is installed
  if (!is_object($media)) {
    return false;
  }

  // only call the database if it is not already loaded
  if (!array_key_exists($media->fid, $paths)) {
    $paths[$media->fid] = mapi_get_workpath($workspace, 'base') .'/'. db_result(db_query("SELECT path FROM {media_folders} WHERE fid=%d", $media->fid));
  }

  return $paths[$media->fid];
}

/**
 * Allows basic where results be executed.
 *
 * @param $where
 *   An SQL where clause
 * @param $order
 *   An order by clause
 * @param $values
 *   The values required to be passed to the query.
 * @param $refresh
 *   List of mids matching where clauses.
 */
function media_item_find($where, $order, $values = array()) {
  if (!is_string($where) && !is_string($order)) {
    return array();
  }

  // NEEDS WORK: Really do some harsh checking of the $where and $order variables
  // try to stop potential SQL injections
  if (strpos($where, ';') !== false || strpos($order, ';') !== false) {
    return array();
  }

  $list = array();
  $sql = 'SELECT mid FROM {media}';
  if ($where) {
    $sql .= ' WHERE '. $where;
  }
  if ($order) {
    $sql .= ' ORDER BY '. $order;
  }

  $result = db_query($sql, $values);
  while ($object = db_fetch_object($result)) {
    $list[] = $object->mid;
  }

  return $list;
}
