<?php

/**
 *  Basic form processing for a media file.
 */
function _media_form($edit = null) {
  global $form_values;

  $form['#media'] = $edit;

  // standard values
  $form['mid'] = array('#type' => 'value', '#value' => $edit->mid);
  $form['fid'] = array('#type' => 'value', '#value' => $edit->fid);
  $form['uid'] = array('#type' => 'value', '#value' => $edit->uid);
  $form['type'] = array('#type' => 'value', '#value' => $edit->type);

  // so we can check it hasn't already been changed
  $form['changed'] = array('#type' => 'hidden', '#value' => $edit->changed);

  // define title
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => true,
    '#maxlength' => 255,
    '#default_value' => $edit->title,
    '#weight' => -10
  );

  // deal with filenames
  $form['filename'] = array('#type' => 'value', '#value' => $edit->filename);

  // upload capability
  if ($edit->filename) {
    // show the default for full.
    $form['media'] = array('#value' => mapi_display($edit->filename, array('context' => 'full')), '#weight' => -8);
  }
  else {
    $form['media'] = array(
      '#type' => 'file',
      '#title' => t('Upload'),
      '#description' => t('Select a file to upload.'),
      '#weight' => -8
    );

    // uploadable form
    $form['#attributes'] = array('enctype' => 'multipart/form-data');
  }

  // metadata information
  $form['metadata'] = array(
    '#type' => 'fieldset',
    '#title' => t('Metadata'),
    '#collapsible' => true,
    '#collapsed' => false,
    '#tree' => true
  );

  // get the form details for the metadata
  if ($meta = media_invoke_metadata('form', $edit->metadata, $edit)) {
    $form['metadata'] = array_merge($form['metadata'], $meta);
  }

  // make sure there is a preview shown if it exists
  $form['#after_build'] = array('_media_form_preview');

  return $form;
}

function _media_form_process($form_values, $temp = null, $workspace = 'media') {
  if (($file = file_save_upload('media', file_directory_temp()))) {
    // make sure that the temporary directory is existant.
    if (!$temp) {
      $temp = media_directory_temp();
    }

    // moves all files uploaded to the temporary directory
    $name = mapi_file_name($file->filepath);
    $ext = mapi_file_ext($file->filepath);

    $type = $form_values['type'];
    $ftype = mapi_extension_type($ext);

    if ($type && $type != $ftype) {
      file_delete($file->filepath);
      form_set_error('media', t('Invalid media type must be of type [%type].', array('%type' => $type)));
      return false;
    }

    // rename potentially problematic characters
    $name = preg_replace('|[^\w\d\-\_\.]+|', '_', $name);

    // rename potentially exploitable files
    if (in_array($ext, array('php', 'pl', 'py', 'cgi', 'asp', 'js', 'htm', 'html'))) {
      $ext .= '_';
    }

    // now that we have it safely named it
    if (!($file->filepath = mapi_file_move($file->filepath, $temp . $name .'.'. $ext, FILE_EXISTS_RENAME, $workspace))) {
      mapi_error(t('Unable to rename file correctly.'));
      return false;
    }

    // remove previous views
    if ($filename = mapi_persist_get($form_values['form_token'])) {
      file_delete($filename);
    }

    // save the preview
    mapi_persist_set($form_values['form_token'], $file->filepath);
  }

  // check for a preview if no media id provided
  if (!$form_values['mid'] && !mapi_persist_get($form_values['form_token'])) {
    form_set_error('media', t('A media file is required.'));
  }

  // check metadata
  $meta = _media_form_flatten($form_values['metadata']);
  media_invoke_metadata('validate', $meta);
}

function _media_form_post($form_values, $workspace = 'media') {
  // should take final thing and create/update media file from it.
  $mid = $form_values['mid'];
  if ($mid) {
    $media = media_item_load($mid);
		if ($media->mid && ($media->title != $form_values['title'])) {
			media_item_rename($media->mid, $form_values['title'], $workspace);
		}
  }
  else {
    $parent = ($form_values['parent'] ? $form_values['parent'] : 0);
    $media = media_item_create(mapi_persist_get($form_values['form_token']), $form_values['title'], $parent);
  }

  // modify and save the metadata
  $meta = _media_form_flatten($form_values['metadata']);
  $meta['title'] = $form_values['title'];
  media_metadata_save($media->mid, $meta);

  // remove any possible previews for form
  if ($filename = mapi_persist_get($form_values['form_token'])) {
    mapi_file_delete($filename, $workspace);
    mapi_persist_set($form_values['form_token'], null);
  }

  return $media;
}

function _media_form_preview($form) {
  global $form_values;

  $op = isset($form_values['op']) ? $form_values['op'] : '';
  if ($op == t('Preview')) {
    drupal_validate_form($form['form_id']['#value'], $form);
    if (!form_get_errors()) {
      // first see if something already exists, otherwise try the form itself
      if (($filename = $form['filename']['#value']) || ($filename = mapi_persist_get($form_values['form_token']))) {
        $form['media']['#description']  = '<strong>'. t('Uploading another file will cancel this one.') .'</strong>';
        $form['media']['#suffix'] = '<div class="media-preview">'. mapi_display($filename) .'</div>';
      }
    }
  }
  return $form;
}

/**
 *  Used for flattening out the form_values of the metadata
 */
function _media_form_flatten($form_values) {
  $list = array();
  if (is_array($form_values)) {
    foreach ($form_values as $key => $value) {
      if (is_array($value)) {
        $list = array_merge($list, _media_form_flatten($value));
      }
      else {
        $list[$key] = $value;
      }
    }
  }
  return $list;
}